@@ -74,10 +74,26 @@ cheese_gconf_get_property (GObject *object, guint prop_id, GValue *value,
       tmp = list;
       while (tmp != NULL)
       {
+        char *str = tmp->data;
+        int j;
+        str[0] = g_ascii_toupper (str[0]);
+        for (j = 1; j < g_utf8_strlen (str, -1); j++)
+        {
+          if (str[j] == '-')
+          {
+            str[j] = ' ';
+            str[j + 1] = g_ascii_toupper (str[j + 1]);
+          }
+          else if (str[j] == '_')
+          {
+            str[j] = '/';
+            str[j + 1] = g_ascii_toupper (str[j + 1]);
+          }
+        }
         if (effects == NULL)
-          effects = tmp->data;
+          effects = str;
         else
-          effects = g_strjoin (",", effects, tmp->data, NULL);
+          effects = g_strjoin (",", effects, str, NULL);
 
         tmp = g_slist_next (tmp);
       }
@@ -106,6 +122,11 @@ cheese_gconf_get_property (GObject *object, guint prop_id, GValue *value,
                                                     CHEESE_GCONF_PREFIX "/photo_path",
                                                     NULL));
       break;
+    case GCONF_PROP_ENABLE_DELETE:
+      g_value_set_boolean (value, gconf_client_get_bool (priv->client,
+							 CHEESE_GCONF_PREFIX "/enable_delete",
+							 NULL));
+      break;      
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -146,11 +167,29 @@ cheese_gconf_set_property (GObject *object, guint prop_id, const GValue *value,
       }
       else
       {
-        effects = g_strsplit (g_value_get_string (value), ",", 12);
+        char *str = g_value_dup_string (value);
+        int j;
+        for (j = 0; j < g_utf8_strlen (str, -1); j++)
+        {
+          if (g_ascii_isupper (str[j]))
+          {
+            str[j] = g_ascii_tolower (str[j]);
+          }
+          else if (str[j] == ' ')
+          {
+            str[j] = '-';
+          }
+          else if (str[j] == '/')
+          {
+            str[j] = '_';
+          }
+        }
+        effects = g_strsplit (str, ",", 12);
         for (i = 0; effects[i] != NULL; i++)
         {
           list = g_slist_append (list, effects[i]);
         }
+        g_free (str);
       }
       gconf_client_set_list (priv->client,
                                CHEESE_GCONF_PREFIX "/selected_effects",
@@ -184,6 +223,12 @@ cheese_gconf_set_property (GObject *object, guint prop_id, const GValue *value,
                           g_value_get_string (value),
                           NULL);
       break;
+    case GCONF_PROP_ENABLE_DELETE:
+      gconf_client_set_bool (priv->client,
+			     CHEESE_GCONF_PREFIX "/enable_delete",
+			     g_value_get_boolean (value),
+			     NULL);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -261,6 +306,14 @@ cheese_gconf_class_init (CheeseGConfClass *klass)
                                                         NULL,
                                                         "",
                                                         G_PARAM_READWRITE));
+
+  g_object_class_install_property (object_class, GCONF_PROP_ENABLE_DELETE,
+                                   g_param_spec_boolean ("gconf_prop_enable_delete",
+							 NULL,
+							 NULL,
+							 FALSE,
+							 G_PARAM_READWRITE));
+
                                                           
   g_type_class_add_private (klass, sizeof (CheeseGConfPrivate));
 }
