@@ -48,7 +48,9 @@ enum
   GCONF_PROP_X_RESOLUTION,
   GCONF_PROP_Y_RESOLUTION,
   GCONF_PROP_VIDEO_PATH,
-  GCONF_PROP_PHOTO_PATH
+  GCONF_PROP_PHOTO_PATH,
+  GCONF_PROP_ENABLE_DELETE
+  
 };
 
 GType        cheese_gconf_get_type (void);
