@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2007,2008 Jaap Haitsma <jaap@haitsma.org>
  * Copyright (C) 2007,2008 daniel g. siegel <dgsiegel@gmail.com>
+ * Copyright (C) 2008 Ryan Zeigler <zeiglerr@gmail.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -25,12 +26,12 @@
 #include <glib-object.h>
 #include <glib.h>
 #include <glib/gstring.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gst/gst.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <X11/Xlib.h>
-#include <X11/extensions/xf86vmode.h>
 #include <libhal.h>
 
 #include "cheese-webcam.h"
@@ -48,24 +49,13 @@ enum CheeseWebcamError
   CHEESE_WEBCAM_ERROR_ELEMENT_NOT_FOUND
 };
 
-typedef struct
-{
-  char *video_device; 
-  char *gstreamer_src;
-  char *product_name; 
-  int num_video_formats;
-  GArray *video_formats;
-} CheeseWebcamDevice;
-
-
 typedef struct
 {
   GtkWidget* video_window;
 
   GstElement *pipeline; 
   GstBus     *bus;
-  /* We build the active pipeline by linking the appropriate pipelines listed
-     below*/
+  /* We build the active pipeline by linking the appropriate pipelines listed below*/
   GstElement *webcam_source_bin;
   GstElement *video_display_bin;
   GstElement *photo_save_bin;
@@ -210,14 +200,14 @@ cheese_webcam_photo_data_cb (GstElement *element, GstBuffer *buffer,
   gst_structure_get_int (structure, "height", &height);
 
   stride = buffer->size/height;
-  pixbuf = gdk_pixbuf_new_from_data(GST_BUFFER_DATA (buffer), GDK_COLORSPACE_RGB, 
-                                    FALSE, bits_per_pixel, width, height, stride, 
-                                    NULL, NULL);
+  pixbuf = gdk_pixbuf_new_from_data (GST_BUFFER_DATA (buffer), GDK_COLORSPACE_RGB, 
+                                     FALSE, bits_per_pixel, width, height, stride, 
+                                     NULL, NULL);
 
   gdk_pixbuf_save (pixbuf, priv->photo_filename, "jpeg", NULL, NULL);
   g_object_unref (G_OBJECT (pixbuf));
 
-  g_signal_handler_disconnect (G_OBJECT(priv->photo_sink), 
+  g_signal_handler_disconnect (G_OBJECT (priv->photo_sink), 
                                priv->photo_handler_signal_id);
 
   g_signal_emit (webcam, webcam_signals[PHOTO_SAVED], 0);
@@ -228,12 +218,11 @@ cheese_webcam_bus_message_cb (GstBus *bus, GstMessage *message, CheeseWebcam *we
 {
   CheeseWebcamPrivate* priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
 
-  if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_EOS)
+  if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_EOS)
   {
+    g_print ("Received EOS message\n");
     g_signal_emit (webcam, webcam_signals [VIDEO_SAVED], 0);
-
-    gst_element_set_locked_state (priv->video_source, FALSE);
-    gst_element_set_locked_state (priv->audio_source, FALSE);
+    
     cheese_webcam_change_sink (webcam, priv->video_display_bin, 
                                priv->photo_save_bin, priv->video_save_bin);
     priv->is_recording = FALSE;
@@ -274,10 +263,10 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
 
   if (!libhal_ctx_init (hal_ctx, &error)) 
   {
-    if (dbus_error_is_set(&error)) 
+    if (dbus_error_is_set (&error)) 
     {
       g_error ("error: libhal_ctx_init: %s: %s\n", error.name, error.message);
-      dbus_error_free(&error);
+      dbus_error_free (&error);
     }
     g_error ("Could not initialise connection to hald.\n"
              "Normally this means the HAL daemon (hald) is not running or not ready");
@@ -289,7 +278,7 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
   if (dbus_error_is_set (&error)) 
   {
     g_error ("error: %s: %s\n", error.name, error.message);
-    dbus_error_free(&error);
+    dbus_error_free (&error);
     return;
   }
 
@@ -310,6 +299,7 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
   {
     priv->webcam_devices[i].num_video_formats = 0;
     priv->webcam_devices[i].video_formats = g_array_new (FALSE, FALSE, sizeof (CheeseVideoFormat));
+    priv->webcam_devices[i].hal_udi = g_strdup (udis[i]);
   }
 
   for (i = 0; i < priv->num_webcam_devices; i++) 
@@ -320,7 +310,7 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
     if (dbus_error_is_set (&error)) 
     {
       g_error ("error: %s: %s\n", error.name, error.message);
-      dbus_error_free(&error);
+      dbus_error_free (&error);
       return;
     }
     priv->webcam_devices[i].video_device = g_strdup (device);
@@ -690,12 +680,12 @@ cheese_webcam_create_webcam_source_bin (CheeseWebcam *webcam)
     priv->current_format = format;
     g_free (resolution);
     
-    find_highest_framerate (format, &framerate_numerator, 
-                            &framerate_denominator);
-    
     if (format == NULL)
       goto fallback;
 
+    find_highest_framerate (format, &framerate_numerator, 
+                            &framerate_denominator);
+
     webcam_input = g_strdup_printf ("%s name=video_source device=%s ! capsfilter name=capsfilter caps=%s,width=%d,height=%d,framerate=%d/%d ! identity",
                                     selected_webcam->gstreamer_src,
                                     selected_webcam->video_device,
@@ -742,7 +732,34 @@ fallback:
 static void 
 cheese_webcam_set_error_element_not_found (GError **error, const char *factoryname)
 {
-  g_set_error (error, CHEESE_WEBCAM_ERROR, CHEESE_WEBCAM_ERROR_ELEMENT_NOT_FOUND, "The element '%s' could not be found.", factoryname);
+  if (error == NULL)
+    return;
+
+  if (*error == NULL)
+  {
+    g_set_error (error, CHEESE_WEBCAM_ERROR, CHEESE_WEBCAM_ERROR_ELEMENT_NOT_FOUND, "%s.", factoryname);
+  }
+  else
+  {
+    /* Ensure that what is found is not a substring of an element; all strings
+     * should have a ' ' or nothing to the left and a '.' or ',' to the right */
+    gchar *found = g_strrstr((*error)->message, factoryname);
+    gchar prev = 0;
+    gchar next = 0;
+
+    if (found != NULL)
+    {
+      prev = *(found - 1);
+      next = *(found + strlen(factoryname));
+    }
+
+    if (found == NULL || 
+        ((found != (*error)->message && prev != ' ') && /* Prefix check */
+         (next != ',' && next != '.'))) /* Postfix check */
+    {
+      g_prefix_error (error, "%s, ", factoryname);
+    }
+  }
 }
 
 static gboolean
@@ -760,48 +777,45 @@ cheese_webcam_create_video_display_bin (CheeseWebcam *webcam, GError **error)
 
   if ((priv->effect_filter = gst_element_factory_make ("identity", "effect")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "identity");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "identity");
   }
   if ((priv->csp_post_effect = gst_element_factory_make ("ffmpegcolorspace", "csp_post_effect")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "ffmpegcolorspace");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "ffmpegcolorspace");
   }
 
   if ((tee = gst_element_factory_make ("tee", "tee")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "tee");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "tee");
   }
   
   if ((save_queue = gst_element_factory_make ("queue", "save_queue")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "queue");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "queue");
   }
 
   if ((video_display_queue = gst_element_factory_make ("queue", "video_display_queue")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "queue");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "queue");
   }
 
   if ((video_scale = gst_element_factory_make ("videoscale", "video_scale")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "videoscale");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "videoscale");
+  }
+  else
+  {
+    /* Use bilinear scaling */
+    g_object_set (video_scale, "method", 1, NULL);
   }
-
-  /* Use bilinear scaling */
-  g_object_set (video_scale, "method", 1, NULL);
 
   if ((video_sink = gst_element_factory_make ("gconfvideosink", "video_sink")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "gconfvideosink");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "gconfvideosink");
   }
 
+  if (error != NULL && *error != NULL)
+    return FALSE;
 
   gst_bin_add_many (GST_BIN (priv->video_display_bin), priv->webcam_source_bin, 
                     priv->effect_filter, priv->csp_post_effect, tee, save_queue, 
@@ -839,15 +853,16 @@ cheese_webcam_create_photo_save_bin (CheeseWebcam *webcam, GError **error)
 
   if ((csp_photo_save_bin = gst_element_factory_make ("ffmpegcolorspace", "csp_photo_save_bin")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "ffmpegcolorspace");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "ffmpegcolorspace");
   }
   if ((priv->photo_sink = gst_element_factory_make ("fakesink", "photo_sink")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "fakesink");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "fakesink");
   }
 
+  if (error != NULL && *error != NULL)
+    return FALSE;
+
   gst_bin_add_many (GST_BIN (priv->photo_save_bin), csp_photo_save_bin,
                     priv->photo_sink, NULL);
 
@@ -877,7 +892,7 @@ cheese_webcam_create_video_save_bin (CheeseWebcam *webcam, GError **error)
   CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
 
   GstElement *audio_queue, *audio_convert, *audio_enc;
-  GstElement *video_save_csp, *video_save_scale, *video_enc;
+  GstElement *video_save_csp, *video_save_rate, *video_save_scale, *video_enc;
   GstElement *mux;
   GstPad *pad;
   gboolean ok;
@@ -886,57 +901,62 @@ cheese_webcam_create_video_save_bin (CheeseWebcam *webcam, GError **error)
 
   if ((priv->audio_source = gst_element_factory_make ("gconfaudiosrc", "audio_source")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "gconfaudiosrc");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "gconfaudiosrc");
   }
   if ((audio_queue = gst_element_factory_make ("queue", "audio_queue")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "queue");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "queue");
   }
   if ((audio_convert = gst_element_factory_make ("audioconvert", "audio_convert")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "audioconvert");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "audioconvert");
   }
   if ((audio_enc = gst_element_factory_make ("vorbisenc", "audio_enc")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "vorbisenc"); 
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "vorbisenc"); 
   }
 
   if ((video_save_csp = gst_element_factory_make ("ffmpegcolorspace", "video_save_csp")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "ffmpegcolorspace");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "ffmpegcolorspace");
   }
   if ((video_enc = gst_element_factory_make ("theoraenc", "video_enc")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "theoraenc");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "theoraenc");
+  }
+  else
+  {
+    g_object_set (video_enc, "keyframe-force", 1, NULL);
+  }
+
+  if ((video_save_rate = gst_element_factory_make ("videorate", "video_save_rate")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found (error, "videorate");
   }
-  g_object_set (video_enc, "keyframe-force", 1, NULL);
   if ((video_save_scale = gst_element_factory_make ("videoscale", "video_save_scale")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "videoscale");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "videoscale");
+  }
+  else
+  {
+    /* Use bilinear scaling */
+    g_object_set (video_save_scale, "method", 1, NULL);
   }
-  /* Use bilinear scaling */
-  g_object_set (video_save_scale, "method", 1, NULL);
 
   if ((mux = gst_element_factory_make ("oggmux", "mux")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "oggmux");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "oggmux");
   }
   if ((priv->video_file_sink = gst_element_factory_make ("filesink", "video_file_sink")) == NULL)
   {
-    cheese_webcam_set_error_element_not_found(error, "filesink");
-    return FALSE;
+    cheese_webcam_set_error_element_not_found (error, "filesink");
   }
 
+  if (error != NULL && *error != NULL)
+    return FALSE;
+
   gst_bin_add_many (GST_BIN (priv->video_save_bin), priv->audio_source, audio_queue,
-                    audio_convert, audio_enc, video_save_csp, video_save_scale, video_enc, 
+                    audio_convert, audio_enc, video_save_csp, video_save_rate, video_save_scale, video_enc, 
                     mux, priv->video_file_sink, NULL);
 
   /* add ghostpad */
@@ -948,7 +968,7 @@ cheese_webcam_create_video_save_bin (CheeseWebcam *webcam, GError **error)
   ok = gst_element_link_many (priv->audio_source, audio_queue, audio_convert, 
                               audio_enc, mux, priv->video_file_sink, NULL);
 
-  ok &= gst_element_link_many (video_save_csp, video_save_scale, video_enc,
+  ok &= gst_element_link_many (video_save_csp, video_save_rate, video_save_scale, video_enc,
                                NULL);
   ok &= gst_element_link (video_enc, mux);
 
@@ -965,6 +985,63 @@ cheese_webcam_get_num_webcam_devices (CheeseWebcam *webcam)
   return priv->num_webcam_devices;
 }
 
+gboolean     
+cheese_webcam_switch_webcam_device (CheeseWebcam *webcam)
+{
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+  gboolean was_recording = FALSE;
+  gboolean pipeline_was_playing = FALSE; 
+  gboolean disp_bin_created = FALSE;
+  gboolean disp_bin_added = FALSE;
+  gboolean disp_bin_linked = FALSE;
+  GError *error = NULL;
+    
+  if (priv->is_recording)
+  {
+    cheese_webcam_stop_video_recording (webcam);
+    was_recording = TRUE;
+  }
+  
+  if (priv->pipeline_is_playing)
+  {
+    cheese_webcam_stop (webcam);
+    pipeline_was_playing = TRUE;
+  }
+  
+  gst_bin_remove (GST_BIN (priv->pipeline), priv->video_display_bin);
+  
+  disp_bin_created = cheese_webcam_create_video_display_bin (webcam, &error);
+  if (!disp_bin_created)
+  {
+    return FALSE;
+  }
+  disp_bin_added = gst_bin_add (GST_BIN (priv->pipeline), priv->video_display_bin);
+  if (!disp_bin_added) 
+  {
+    gst_object_sink (priv->video_display_bin);
+    return FALSE;
+  }
+  
+  disp_bin_linked = gst_element_link (priv->video_display_bin, priv->photo_save_bin);
+  if (!disp_bin_linked)
+  {
+    gst_bin_remove (GST_BIN (priv->pipeline), priv->video_display_bin);
+    return FALSE;
+  }
+  
+  if (pipeline_was_playing)
+  {
+    cheese_webcam_play (webcam);
+  }
+  
+  /* if (was_recording)
+   {
+     Restart recording... ?
+   } */
+  
+  return TRUE;
+}
+
 void
 cheese_webcam_play (CheeseWebcam *webcam)
 {
@@ -1044,7 +1121,7 @@ cheese_webcam_set_effect (CheeseWebcam *webcam, CheeseWebcamEffect effect)
   effect_filter = gst_parse_bin_from_description (effects_pipeline_desc, TRUE, &err);
   if (!effect_filter || (err != NULL))
   {
-    g_error_free(err);
+    g_error_free (err);
     g_error ("ERROR effect_filter\n");
   }
   cheese_webcam_change_effect_filter (webcam, effect_filter);
@@ -1071,10 +1148,8 @@ cheese_webcam_stop_video_recording (CheeseWebcam *webcam)
   CheeseWebcamPrivate* priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
 
   /* Send EOS message down the pipeline by stopping video and audio source*/
-  gst_element_set_state (priv->video_source, GST_STATE_NULL);
-  gst_element_set_locked_state (priv->video_source, TRUE);
-  gst_element_set_state (priv->audio_source, GST_STATE_NULL);
-  gst_element_set_locked_state (priv->audio_source, TRUE);
+  gst_element_send_event (priv->video_source, gst_event_new_eos ());
+  gst_element_send_event (priv->audio_source, gst_event_new_eos ());
 }
 
 void
@@ -1119,6 +1194,7 @@ cheese_webcam_finalize (GObject *object)
       g_free (g_array_index (priv->webcam_devices[i].video_formats, CheeseVideoFormat, j).framerates);
       g_free (g_array_index (priv->webcam_devices[i].video_formats, CheeseVideoFormat, j).mimetype);
     }
+    g_free (priv->webcam_devices[i].hal_udi);
     g_array_free (priv->webcam_devices[i].video_formats, TRUE);
   }
   g_free (priv->webcam_devices);
@@ -1168,8 +1244,8 @@ cheese_webcam_set_property (GObject *object, guint prop_id, const GValue *value,
   {
     case PROP_VIDEO_WINDOW:
       priv->video_window = g_value_get_pointer (value);
-      g_signal_connect(priv->video_window, "expose-event", 
-                       G_CALLBACK(cheese_webcam_expose_cb), self);
+      g_signal_connect (priv->video_window, "expose-event", 
+                        G_CALLBACK(cheese_webcam_expose_cb), self);
       break;
     case PROP_DEVICE_NAME:
       g_free (priv->device_name);
@@ -1289,36 +1365,33 @@ cheese_webcam_new (GtkWidget* video_window, char *webcam_device_name,
 }
 
 void
-cheese_webcam_setup (CheeseWebcam *webcam, GError **error)
+cheese_webcam_setup (CheeseWebcam *webcam, char *hal_dev_udi, GError **error)
 {
   CheeseWebcamPrivate* priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
   gboolean ok = TRUE;
   GError *tmp_error = NULL;
 
   cheese_webcam_detect_webcam_devices (webcam);
-  cheese_webcam_create_video_display_bin (webcam, &tmp_error);
-  if (tmp_error != NULL)
+  
+  if (hal_dev_udi != NULL)
   {
-    g_propagate_error (error, tmp_error);
-    return;
+    cheese_webcam_set_device_by_dev_udi (webcam, hal_dev_udi);
   }
 
+  priv->pipeline = gst_pipeline_new ("pipeline");
+
+  cheese_webcam_create_video_display_bin (webcam, &tmp_error);
+
   cheese_webcam_create_photo_save_bin (webcam, &tmp_error);
-  if (tmp_error != NULL)
-  {
-    g_propagate_error (error, tmp_error);
-    return;
-  }
 
   cheese_webcam_create_video_save_bin (webcam, &tmp_error);
   if (tmp_error != NULL)
   {
     g_propagate_error (error, tmp_error);
+    g_prefix_error (error, _("One or more needed gstreamer elements are missing: "));
     return;
   }
 
-  priv->pipeline = gst_pipeline_new ("pipeline");
-
   gst_bin_add_many (GST_BIN (priv->pipeline), priv->video_display_bin, 
                     priv->photo_save_bin, NULL);
 
@@ -1335,6 +1408,50 @@ cheese_webcam_setup (CheeseWebcam *webcam, GError **error)
 
 }
 
+int         
+cheese_webcam_get_selected_device_index (CheeseWebcam *webcam)
+{
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+  return priv->selected_device;
+}
+
+GArray *
+cheese_webcam_get_webcam_devices (CheeseWebcam *webcam)
+{
+  GArray *devices_arr;
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+
+  devices_arr = g_array_sized_new (FALSE,
+                                   TRUE,
+                                   sizeof (CheeseWebcamDevice),
+                                   priv->num_webcam_devices);
+  devices_arr = g_array_append_vals (devices_arr,
+                                     priv->webcam_devices,
+                                     priv->num_webcam_devices);
+  return devices_arr;
+}
+
+void
+cheese_webcam_set_device_by_dev_file (CheeseWebcam *webcam, char *file)
+{
+  g_object_set (webcam, "device_name", file, NULL);
+}
+
+void
+cheese_webcam_set_device_by_dev_udi (CheeseWebcam *webcam, char *udi)
+{
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+  int i;
+  for (i = 0; i < priv->num_webcam_devices; i++)
+  {
+    if (strcmp (priv->webcam_devices[i].hal_udi, udi) == 0)
+    {
+      g_object_set (webcam, "device_name", priv->webcam_devices[i].video_device, NULL);
+      break;
+    }
+  }
+}
+
 GArray *
 cheese_webcam_get_video_formats (CheeseWebcam *webcam)
 {
@@ -1377,4 +1494,3 @@ cheese_webcam_get_current_video_format (CheeseWebcam *webcam)
     
   return priv->current_format;
 }
-
